import Event from './event';
import Key from './key';
import Subscriber from './subscriber';
import Transition from './transition';
import TransitionPredicate from './transition-predicate';
import TransitionRecord from './transition-record';
export default class StateMachine {
    static create(initialState: Key, events?: Event[]): StateMachine;
    private history;
    private subscribers;
    private transitioner;
    private transitions;
    constructor(initialState: Key, events?: Event[]);
    getState(): Key;
    getPreviousState(): Key | undefined;
    getHistory(): {
        index: number;
        stack: TransitionRecord[];
    };
    addEvent(name: Key, transitions: Transition[]): void;
    addEvents(events: Event[]): void;
    removeEvent(name: Key): void;
    removeEvents(names: Key[]): void;
    hasEvent(name: Key): boolean;
    addTransition(transition: Transition): void;
    addTransitions(transitions: Transition[]): void;
    removeTransition(transition: Transition): void;
    removeTransitions(transitions: Transition[]): void;
    hasTransition(predicate: TransitionPredicate): boolean;
    canTransition(state: Key): boolean;
    transition(state: Key, data?: any): void;
    triggerEvent(event: Key, data?: any): void;
    undoTransition(): void;
    redoTransition(): void;
    canUndo(): boolean;
    canRedo(): boolean;
    subscribe(subscriber: Subscriber): void;
    unsubscribe(subscriber: Subscriber): void;
}
